// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/ChargedFinalState.hh"

namespace Rivet {


  class ATLAS_2011_I929691 : public Analysis {
  public:

    /// Constructor
    DEFAULT_RIVET_ANALYSIS_CTOR(ATLAS_2011_I929691);

    void init() {
      const FinalState FS(Cuts::abseta < 2.0);
      
      FastJets ANTIKT_06_JETS(FS, FastJets::ANTIKT, 0.6, JetAlg::NO_MUONS, JetAlg::NO_INVISIBLES);
      declare(ANTIKT_06_JETS, "jets");
	  
      ChargedFinalState TRACKS(Cuts::pT > 0.5*GeV && Cuts::abseta < 2.0);
      declare(TRACKS, "tracks");
      
      // Set up the histograms (each element is a binning in jet pT)
      for (int i = 0; i < 10; i++) {
        _p_F_z[i]     = bookProfile1D(i+1, 1, 1);
        _p_rho_r[i]   = bookProfile1D(i+11, 1, 1);
        _p_f_pTrel[i] = bookProfile1D(i+21, 1, 1);
      }

    }

    void analyze(const Event& event) {
      const double weight = event.weight();
      
      for (int i = 0; i < 10; ++i) {
	
        const Jets& jets = apply<FastJets>(event, "jets").jetsByPt(Cuts::pT > bedges[i] && Cuts::pT < bedges[i+1] && Cuts::absrap < 1.2);
        const Particles& tracks = apply<ChargedFinalState>(event, "tracks").particlesByPt();
        
        int n_jets = jets.size();
        if (!n_jets)  continue;

        // First... count the tracks
        Histo1D *_h_ntracks_z     = new Histo1D(*_p_F_z[i]);
        Histo1D *_h_ntracks_r     = new Histo1D(*_p_rho_r[i]);
        Histo1D *_h_ntracks_pTrel = new Histo1D(*_p_f_pTrel[i]);

        foreach (const Jet& j, jets) {
          foreach (const Particle& p, tracks) {
            if (r(j, p) > 0.6)  continue; // The paper uses pseudorapidity, but this is a requirement for filling the histogram
            _h_ntracks_z->fill(z(j, p), 1.0/n_jets);
	          _h_ntracks_r->fill(r(j, p), 1.0/n_jets);
	          _h_ntracks_pTrel->fill(pTrel(j, p), 1.0/n_jets);
          }
        }
	
       // Then... calculate the observable and fill the profiles
       foreach (const HistoBin1D& b, _h_ntracks_z->bins()) {
	       _p_F_z[i]->fill(b.xMid(), b.height(), weight);
       }
       foreach (const HistoBin1D& b, _h_ntracks_r->bins()) {
         _p_rho_r[i]->fill(b.xMid(), b.area()/annulus_area(b.xMin(), b.xMax()), weight);
       }
       foreach (const HistoBin1D& b, _h_ntracks_pTrel->bins()) {
          _p_f_pTrel[i]->fill(b.xMid(), b.height(), weight);
        }
      }      
    }

    void finalize() {  }

  private:

    Profile1DPtr _p_F_z[10], _p_rho_r[10], _p_f_pTrel[10];
    vector<double> bedges = { 25., 40., 60., 80., 110., 160., 210., 260., 310., 400., 500. };
    
    double z (const Jet& jet, const Particle& ch) {
      return jet.p3().dot(ch.p3())/jet.p3().mod2();
    }
    
    double r (const Jet& j, const Particle& ch) {
      return deltaR(j, ch, RAPIDITY);
    }
    
    double pTrel (const Jet& jet, const Particle& ch) {
      return (ch.p3().cross(jet.p3())).mod()/(jet.p3().mod());
    }
	
    // To calculate the area of the annulus in an r bin
    double annulus_area(double r1, double r2) {
      return M_PI*(pow(r2, 2) - pow(r1, 2));
    }

  };

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2011_I929691);

}
